require 'test_helper'
require 'schmetterling/around'

class AroundTest < Test::Unit::TestCase

  def setup
    @calls = []
    @adviced_class = Class.new do
      def bar(calls)
        calls << :method_called
      end
    end
  end

  def test_executes_the_advice_around_the_adviced_method
    advice = -> (original_method, object, calls) {
      calls << :before_called
      original_method.call(calls)
      calls << :after_called
    }
    Schmetterling::Around.new(@adviced_class, :bar, advice)
    @adviced_class.new.bar(@calls)
    assert_equal [:before_called, :method_called, :after_called], @calls
  end

  def test_works_with_blocks
    Schmetterling::Around.new(@adviced_class, :bar) do |original_method, object, calls|
      calls << :before_called
      original_method.call(calls)
      calls << :after_called
    end
    @adviced_class.new.bar(@calls)
    assert_equal [:before_called, :method_called, :after_called], @calls
  end

  def test_changing_arguments
    @changed_argument = []
    advice = -> (original_method, object, calls) {
      calls << :before_called
      original_method.call(@changed_argument)
      calls << :after_called
    }
    Schmetterling::Around.new(@adviced_class, :bar, advice)
    @adviced_class.new.bar(@calls)
    assert_equal [:before_called, :after_called], @calls
    assert_equal [:method_called], @changed_argument
  end

  def test_skipping_original_method
    advice = -> (original_method, object, calls) {
      calls << :evil_called
    }
    Schmetterling::Around.new(@adviced_class, :bar, advice)
    @adviced_class.new.bar(@calls)
    assert_equal [:evil_called], @calls
  end

  def test_advice_receives_the_adviced_object
    @adviced_object = @adviced_class.new
    Schmetterling::Around.new(@adviced_class, :bar) do |original_method, object, calls|
      assert_equal @adviced_object, object
      original_method.call(calls)
    end
    @adviced_object.bar(@calls)
    assert_equal [:method_called], @calls
  end

end
