require 'test_helper'
require 'schmetterling/before'
require 'schmetterling/after'

class BeforeTest < Test::Unit::TestCase

  def setup
    @calls = []
    @adviced_class = Class.new do
      def bar(calls)
        calls << :method_called
      end
    end
  end

  def test_executes_the_advice_before_adviced_method
    Schmetterling::Before.new(@adviced_class, :bar, -> (object, calls) { calls << :before_called })
    @adviced_class.new.bar(@calls)
    assert_equal [:before_called, :method_called], @calls
  end

  def test_executes_multiple_advices
    Schmetterling::Before.new(@adviced_class, :bar, -> (object, calls) { calls << :before1_called })
    Schmetterling::Before.new(@adviced_class, :bar, -> (object, calls) { calls << :before2_called })
    @adviced_class.new.bar(@calls)
    assert_equal [:before2_called, :before1_called, :method_called], @calls
  end

  def test_works_on_child_classes
    @child_class = Class.new(@adviced_class)
    Schmetterling::Before.new(@adviced_class, :bar, -> (object, calls) { calls << :before_called })
    @child_class.new.bar(@calls)
    assert_equal [:before_called, :method_called], @calls
  end

  def test_works_with_blocks
    Schmetterling::After.new(@adviced_class, :bar) do |object, calls|
      calls << :after_called
    end
    @adviced_class.new.bar(@calls)
    assert_equal [:method_called, :after_called], @calls
  end

  def test_is_always_called_before_the_after_aspects
    Schmetterling::After.new(@adviced_class, :bar, -> (_, calls) { calls << :after1_called })
    Schmetterling::Before.new(@adviced_class, :bar, -> (_, calls) { calls << :before1_called })
    Schmetterling::After.new(@adviced_class, :bar, -> (_, calls) { calls << :after2_called })
    Schmetterling::Before.new(@adviced_class, :bar, -> (_, calls) { calls << :before2_called })
    @adviced_class.new.bar(@calls)
    assert_equal [:before2_called, :before1_called, :method_called, :after1_called, :after2_called], @calls
  end

  def test_advice_receives_the_adviced_object
    @adviced_object = @adviced_class.new
    Schmetterling::Before.new(@adviced_class, :bar) do |object, calls|
      assert_equal @adviced_object, object
      calls << :before_called
    end
    @adviced_object.bar(@calls)
    assert_equal [:before_called, :method_called], @calls
  end

end
